'use strict';

var React = require('react');
var reactNative = require('react-native');
var SyntaxContext = require('./SyntaxContext.js');

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
function log(message, ...args) {
  {
    console.log(`[SyntaxHighlighter] ${message}`, ...args);
  }
}
const defaultHighlightLines = [];
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines
}) => {
  var _a;
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  const [tokens, setTokens] = React.useState([]);
  React.useEffect(() => {
    const highlight = () => __async(void 0, null, function* () {
      try {
        log("Starting highlighting process");
        log("Input:", { text, language, theme: theme.name });
        setLoading(true);
        const highlighter = yield SyntaxContext.getHighlighter();
        log("Highlighter instance obtained");
        if (!text) {
          throw new Error("Text is required");
        }
        if (!language) {
          throw new Error("Language is required");
        }
        if (!theme) {
          throw new Error("Theme is required");
        }
        log("Attempting tokenization...");
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme: theme.name || "default",
          includeExplanation: true
        });
        log("Raw tokenization result:", result);
        if (!result) {
          throw new Error("Tokenization failed, result is undefined");
        }
        if (!result.tokens || !Array.isArray(result.tokens)) {
          throw new Error("Invalid tokens array in result");
        }
        const processedTokens = result.tokens.map((line) => {
          var _a2, _b;
          if (typeof line === "string") {
            return [
              {
                content: line,
                color: ((_a2 = theme.colors) == null ? void 0 : _a2.foreground) || "#ffffff",
                fontStyle: 0,
                offset: 0
                // Add a default offset value
              }
            ];
          }
          if (line.length === 1 && typeof line[0] === "string") {
            return [
              {
                content: line[0],
                color: ((_b = theme.colors) == null ? void 0 : _b.foreground) || "#ffffff",
                fontStyle: 0,
                offset: 0
                // Add a default offset value
              }
            ];
          }
          return line;
        });
        log("Processed tokens:", processedTokens);
        setTokens(processedTokens);
        setError(null);
      } catch (err) {
        const errorMessage = err instanceof Error ? err.message : "Failed to highlight code";
        log("Error during highlighting:", err);
        setError(errorMessage);
        setTokens([]);
      } finally {
        setLoading(false);
      }
    });
    highlight();
  }, [text, language, theme]);
  const renderToken = (token, index) => {
    var _a2;
    log("Rendering token:", token);
    return /* @__PURE__ */ React.createElement(
      reactNative.Text,
      {
        key: `${token.offset || index}-${index}`,
        style: [
          {
            color: token.color || ((_a2 = theme.colors) == null ? void 0 : _a2.foreground) || "#ffffff",
            fontSize,
            fontFamily: "monospace",
            fontStyle: token.fontStyle === 2 ? "italic" : "normal",
            fontWeight: token.fontStyle === 1 ? "bold" : "normal"
          },
          textStyle
        ]
      },
      token.content
    );
  };
  const renderLine = (lineTokens, index) => {
    log(`Rendering line ${index} with ${lineTokens.length} tokens`);
    const isHighlighted = highlightLines.includes(index + 1);
    return /* @__PURE__ */ React.createElement(
      reactNative.View,
      {
        key: index,
        style: [
          {
            flexDirection: "row",
            flexWrap: "wrap",
            backgroundColor: isHighlighted ? "rgba(255,255,255,0.1)" : "transparent"
          }
        ]
      },
      Array.isArray(lineTokens) ? lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex)) : renderToken(lineTokens, 0)
    );
  };
  if (loading) {
    return /* @__PURE__ */ React.createElement(reactNative.View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(reactNative.ActivityIndicator, null));
  }
  if (error) {
    return /* @__PURE__ */ React.createElement(reactNative.View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(reactNative.Text, { style: { color: "#ff6b6b" } }, "Error:", error));
  }
  return /* @__PURE__ */ React.createElement(reactNative.ScrollView, __spreadProps(__spreadValues({}, scrollViewProps), { horizontal: true, style: [{ backgroundColor: ((_a = theme.colors) == null ? void 0 : _a.background) || "#1e1e1e" }, scrollViewProps == null ? void 0 : scrollViewProps.style], contentContainerStyle: [{ padding: 10 }, containerStyle] }), /* @__PURE__ */ React.createElement(reactNative.View, { onStartShouldSetResponder: () => true }, tokens.map((lineTokens, index) => renderLine(lineTokens, index))));
};

exports.SyntaxHighlighter = SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map
