/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import { getHighlighter } from './SyntaxContext'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args)
  }
}

export interface SyntaxHighlighterProps {
  text: string
  language: string
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  highlightLines?: number[]
}

const defaultHighlightLines: number[] = []

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines,
}) => {
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])

  React.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process')
        log('Input:', { text, language, theme: theme.name })

        setLoading(true)
        const highlighter = await getHighlighter()
        log('Highlighter instance obtained')

        // Validate inputs
        if (!text) {
          throw new Error('Text is required')
        }
        if (!language) {
          throw new Error('Language is required')
        }
        if (!theme) {
          throw new Error('Theme is required')
        }

        // Try tokenization with explicit theme name
        log('Attempting tokenization...')
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme: theme.name || 'default',
          includeExplanation: true,
        })

        log('Raw tokenization result:', result)

        if (!result) {
          throw new Error('Tokenization failed, result is undefined')
        }

        if (!result.tokens || !Array.isArray(result.tokens)) {
          throw new Error('Invalid tokens array in result')
        }

        // Break the tokens into lines if they aren't already
        interface ProcessedToken extends ThemedToken {
          content: string
          color: string
          fontStyle: number
        }

        interface ProcessedToken extends ThemedToken {
          content: string
          color: string
          fontStyle: number
        }

        interface ProcessedToken extends ThemedToken {
          content: string
          color: string
          fontStyle: number
        }

        interface TokenizationResult {
          tokens: (ThemedToken | string)[][]
        }

        const processedTokens: ProcessedToken[][] = (result as TokenizationResult).tokens.map((line): ProcessedToken[] => {
          // If the line is just a string, convert it to a token
          if (typeof line === 'string') {
            return [
              {
                content: line,
                color: theme.colors?.foreground || '#ffffff',
                fontStyle: 0,
                offset: 0, // Add a default offset value
              },
            ]
          }

          // If it's a single token spanning multiple lines, split it
          if (line.length === 1 && typeof line[0] === 'string') {
            return [
              {
                content: line[0],
                color: theme.colors?.foreground || '#ffffff',
                fontStyle: 0,
                offset: 0, // Add a default offset value
              },
            ]
          }

          return line as ProcessedToken[]
        })

        log('Processed tokens:', processedTokens)

        setTokens(processedTokens)
        setError(null)
      }
      catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code'
        log('Error during highlighting:', err)
        setError(errorMessage)
        setTokens([])
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    log('Rendering token:', token)
    return (
      <Text
        key={`${token.offset || index}-${index}`}
        style={[
          {
            color: token.color || theme.colors?.foreground || '#ffffff',
            fontSize,
            fontFamily: 'monospace',
            fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
            fontWeight: token.fontStyle === 1 ? 'bold' : 'normal',
          },
          textStyle,
        ]}
      >
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => {
    log(`Rendering line ${index} with ${lineTokens.length} tokens`)
    const isHighlighted = highlightLines.includes(index + 1)

    return (
      <View
        key={index}
        style={[
          {
            flexDirection: 'row',
            flexWrap: 'wrap',
            backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent',
          },
        ]}
      >
        {Array.isArray(lineTokens) ? lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex)) : renderToken(lineTokens, 0)}
      </View>
    )
  }

  if (loading) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: theme.colors?.background || '#1e1e1e' }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 10 }, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}
